<?php

namespace Drupal\book_moderation_sync\Hook;

use Drupal\book\BookOutlineStorageInterface;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\node\NodeInterface;

/**
 * Hooks for book moderation.
 */
class BookModerationHooks {

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ModerationInformationInterface $moderationInformation,
    protected BookOutlineStorageInterface $bookOutlineStorage,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->logger = $logger_factory->get('book_moderation_sync');
  }

  /**
   * Hook for node_presave event.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The book node.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  #[Hook('node_presave')]
  public function nodePresave(NodeInterface $entity) {
    // Check if the entity is a book and is moderated.
    if ($this->moderationInformation->isModeratedEntity($entity)) {
      $this->syncModerationState($entity, $entity->get('moderation_state')->value);
    }
  }

  /**
   * Synchronizes the moderation state with the book's children.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The book node.
   * @param string $state
   *   The new moderation state.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function syncModerationState(NodeInterface $node, $state) {
    $result = $this->bookOutlineStorage->loadBookChildren($node->id());
    $children = $this->entityTypeManager->getStorage('node')->loadMultiple(array_keys($result));

    foreach ($children as $child_entity) {
      if ($child_entity && $this->moderationInformation->isModeratedEntity($child_entity)) {
        $child_entity->set('moderation_state', $state);
        try {
          $child_entity->save();
        }
        catch (EntityStorageException $e) {
          // Log the error if saving fails.
          $this->logger->error('Failed to sync moderation state for child node @nid: @message', [
            '@nid' => $child_entity->id(),
            '@message' => $e->getMessage(),
          ]);
        }
      }
    }
  }

}
